@@ -15,7 +15,8 @@
  *
  *  You should have received a copy of the GNU Library General Public
  *  License along with this library; if not, write to the Free
- *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+ *  MA 02110-1301, USA.
  */
 
 /*
@@ -39,18 +40,36 @@
 #define RLRASTER_H_
 
 
-/* version of the header for the library: 0.21 */
-#define WRASTER_HEADER_VERSION	22
+/* version of the header for the library */
+#define WRASTER_HEADER_VERSION	23
 
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 
-#ifdef XSHM
+#ifdef USE_XSHM
 #include <X11/extensions/XShm.h>
 #endif
 
 
+/*
+ * Define some macro to provide attributes for the compiler
+ *
+ * These attributes help producing better code and/or detecting bugs, however not all compiler support them
+ * as they are not standard.
+ * Because we're a public API header, we can't count on autoconf to detect them for us, so we use that #if
+ * mechanism and define an internal macro appropriately. Please note that the macro are not considered being
+ * part of the public API.
+ */
+#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 5)
+#define __wrlib_deprecated(msg)  __attribute__ ((deprecated(msg)))
+#elif __GNUC__ >= 3
+#define __wrlib_deprecated(msg)  __attribute__ ((deprecated))
+#else
+#define __wrlib_deprecated(msg)
+#endif
+
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -81,30 +100,43 @@ extern "C" {
 
 
 
-
+/* image display modes */
+typedef enum {
+	RDitheredRendering = 0,
+	RBestMatchRendering = 1
+} RRenderingMode;
 
 
 /* std colormap usage/creation modes */
-enum {
-    RUseStdColormap,		       /* default. fallbacks to RIgnore.. if
-    there is none defined */
-    RCreateStdColormap,
-    RIgnoreStdColormap
-};
+typedef enum {
+	RUseStdColormap,	/* default: fallbacks to RIgnore if there is none defined */
+	RCreateStdColormap,
+	RIgnoreStdColormap
+} RStdColormapMode;
 
 
+/* smoothed scaling filter types */
+typedef enum {
+	RBoxFilter,
+	RTriangleFilter,
+	RBellFilter,
+	RBSplineFilter,
+	RLanczos3Filter,
+	RMitchellFilter
+} RScalingFilter;
+
 
 typedef struct RContextAttributes {
     int flags;
-    int render_mode;
+    RRenderingMode render_mode;
     int colors_per_channel;	       /* for PseudoColor */
     float rgamma;		       /* gamma correction for red, */
     float ggamma;		       /* green, */
     float bgamma;		       /* and blue */
     VisualID visualid;	       /* visual ID to use */
     int use_shared_memory;	       /* True of False */
-    int scaling_filter;
-    int standard_colormap_mode;    /* what to do with std cma */
+    RScalingFilter scaling_filter;
+    RStdColormapMode standard_colormap_mode;    /* what to do with std cma */
 } RContextAttributes;
 
 
@@ -146,7 +178,9 @@ typedef struct RContext {
 
     struct {
         unsigned int use_shared_pixmap:1;
-        unsigned int optimize_for_speed:1;
+        unsigned int optimize_for_speed:1
+            __wrlib_deprecated("Flag optimize_for_speed in RContext is not used anymore "
+                               "and will be removed in future version, please do not use");
     } flags;
 } RContext;
 
@@ -204,45 +238,27 @@ typedef struct RXImage {
     XImage *image;
 
     /* Private data. Do not access */
-#ifdef XSHM
+#ifdef USE_XSHM
     XShmSegmentInfo info;
     char is_shared;
 #endif
 } RXImage;
 
 
-/* image display modes */
-enum {
-    RDitheredRendering = 0,
-    RBestMatchRendering = 1
-};
-
-
-/* smoothed scaling filter types */
-enum {
-    RBoxFilter,
-    RTriangleFilter,
-    RBellFilter,
-    RBSplineFilter,
-    RLanczos3Filter,
-    RMitchellFilter
-};
-
-
 /* note that not all operations are supported in all functions */
-enum {
+typedef enum {
     RClearOperation,	       /* clear with 0 */
     RCopyOperation,
     RNormalOperation,	       /* same as combine */
     RAddOperation,
     RSubtractOperation
-};
+} RPixelOperation;
 
 
-enum {
+typedef enum {
     RAbsoluteCoordinates = 0,
     RRelativeCoordinates = 1
-};
+} RCoordinatesMode;
 
 
 enum {
@@ -259,17 +275,25 @@ enum {
 /* 2 pixel width */
 #define RBEV_RAISED3	3
 
-enum {
+typedef enum {
     RHorizontalGradient = 2,
     RVerticalGradient = 3,
     RDiagonalGradient = 4
-};
+} RGradientStyle;
 /* for backwards compatibility */
 #define RGRD_HORIZONTAL  RHorizontalGradient
 #define RGRD_VERTICAL 	RVerticalGradient
 #define RGRD_DIAGONAL	RDiagonalGradient
 
 
+/*
+ * How an image can be flipped, for RFlipImage
+ *
+ * Values are actually bit-mask which can be OR'd
+ */
+#define RHorizontalFlip	0x0001
+#define RVerticalFlip	0x0002
+
 
 /* error codes */
 #define RERR_NONE		0
@@ -289,6 +313,11 @@ enum {
 #define RERR_INTERNAL		128    /* should not happen */
 
 
+/*
+ * Cleaning before application exit
+ */
+void RShutdown(void);
+
 /*
  * Returns a NULL terminated array of strings containing the
  * supported formats, such as: TIFF, XPM, PNG, JPEG, PPM, GIF
@@ -297,17 +326,17 @@ enum {
 char **RSupportedFileFormats(void);
 
 
-char *RGetImageFileFormat(char *file);
+char *RGetImageFileFormat(const char *file);
 
 /*
  * Xlib contexts
  */
 RContext *RCreateContext(Display *dpy, int screen_number,
-                         RContextAttributes *attribs);
+                         const RContextAttributes *attribs);
 
 void RDestroyContext(RContext *context);
 
-Bool RGetClosestXColor(RContext *context, RColor *color, XColor *retColor);
+Bool RGetClosestXColor(RContext *context, const RColor *color, XColor *retColor);
 
 /*
  * RImage creation
@@ -319,7 +348,7 @@ RImage *RCreateImageFromXImage(RContext *context, XImage *image, XImage *mask);
 RImage *RCreateImageFromDrawable(RContext *context, Drawable drawable,
                                  Pixmap mask);
 
-RImage *RLoadImage(RContext *context, char *file, int index);
+RImage *RLoadImage(RContext *context, const char *file, int index);
 
 RImage* RRetainImage(RImage *image);
 
@@ -330,7 +359,7 @@ RImage *RGetImageFromXPMData(RContext *context, char **xpmData);
 /*
  * RImage storing
  */
-Bool RSaveImage(RImage *image, char *filename, char *format);
+Bool RSaveImage(RImage *image, const char *filename, const char *format);
 
 /*
  * Area manipulation
@@ -340,7 +369,7 @@ RImage *RCloneImage(RImage *image);
 RImage *RGetSubImage(RImage *image, int x, int y, unsigned width,
                      unsigned height);
 
-void RCombineImageWithColor(RImage *image, RColor *color);
+void RCombineImageWithColor(RImage *image, const RColor *color);
 
 void RCombineImages(RImage *image, RImage *src);
 
@@ -356,6 +385,9 @@ void RCombineAreaWithOpaqueness(RImage *image, RImage *src, int sx, int sy,
                                 unsigned width, unsigned height, int dx, int dy,
                                 int opaqueness);
 
+void RCombineAlpha(unsigned char *d, unsigned char *s, int s_has_alpha,
+		   int width, int height, int dwi, int swi, int opacity);
+
 RImage *RScaleImage(RImage *image, unsigned new_width, unsigned new_height);
 
 RImage *RSmoothScaleImage(RImage *src, unsigned new_width,
@@ -363,64 +395,69 @@ RImage *RSmoothScaleImage(RImage *src, unsigned new_width,
 
 RImage *RRotateImage(RImage *image, float angle);
 
+RImage *RFlipImage(RImage *image, int mode);
 
 RImage *RMakeTiledImage(RImage *tile, unsigned width, unsigned height);
 
 RImage* RMakeCenteredImage(RImage *image, unsigned width, unsigned height,
-                           RColor *color);
+                           const RColor *color);
 
 /*
  * Drawing
  */
 Bool RGetPixel(RImage *image, int x, int y, RColor *color);
 
-void RPutPixel(RImage *image, int x, int y, RColor *color);
+void RPutPixel(RImage *image, int x, int y, const RColor *color);
 
-void ROperatePixel(RImage *image, int operation, int x, int y, RColor *color);
+void ROperatePixel(RImage *image, RPixelOperation operation, int x, int y, const RColor *color);
 
-void RPutPixels(RImage *image, RPoint *points, int npoints, int mode,
-                RColor *color);
+void RPutPixels(RImage *image, const RPoint *points, int npoints, RCoordinatesMode mode,
+                const RColor *color);
 
-void ROperatePixels(RImage *image, int operation, RPoint *points,
-                    int npoints, int mode, RColor *color);
+void ROperatePixels(RImage *image, RPixelOperation operation, const RPoint *points,
+                    int npoints, RCoordinatesMode mode, const RColor *color);
 
-int RDrawLine(RImage *image, int x0, int y0, int x1, int y1, RColor *color);
+int RDrawLine(RImage *image, int x0, int y0, int x1, int y1, const RColor *color);
 
-int ROperateLine(RImage *image, int operation, int x0, int y0, int x1, int y1,
-                 RColor *color);
+int ROperateLine(RImage *image, RPixelOperation operation, int x0, int y0, int x1, int y1,
+                 const RColor *color);
 
-void RDrawLines(RImage *image, RPoint *points, int npoints, int mode,
-                RColor *color);
+void RDrawLines(RImage *image, const RPoint *points, int npoints, RCoordinatesMode mode,
+                const RColor *color);
 
-void ROperateLines(RImage *image, int operation, RPoint *points, int npoints,
-                   int mode, RColor *color);
+void ROperateLines(RImage *image, RPixelOperation operation, const RPoint *points, int npoints,
+                   RCoordinatesMode mode, const RColor *color);
 
-void RDrawSegments(RImage *image, RSegment *segs, int nsegs, RColor *color);
+void ROperateRectangle(RImage *image, RPixelOperation operation, int x0, int y0, int x1, int y1, const RColor *color);
 
-void ROperateSegments(RImage *image, int operation, RSegment *segs, int nsegs,
-                      RColor *color);
+void RDrawSegments(RImage *image, const RSegment *segs, int nsegs, const RColor *color);
+
+void ROperateSegments(RImage *image, RPixelOperation operation, const RSegment *segs, int nsegs,
+                      const RColor *color);
 
 /*
  * Color convertion
  */
-void RRGBtoHSV(RColor *color, RHSVColor *hsv);
-void RHSVtoRGB(RHSVColor *hsv, RColor *rgb);
+void RRGBtoHSV(const RColor *color, RHSVColor *hsv);
+void RHSVtoRGB(const RHSVColor *hsv, RColor *rgb);
 
 /*
  * Painting
  */
-void RClearImage(RImage *image, RColor *color);
+void RClearImage(RImage *image, const RColor *color);
 
-void RFillImage(RImage *image, RColor *color);
+void RLightImage(RImage *image, const RColor *color);
+
+void RFillImage(RImage *image, const RColor *color);
 
 void RBevelImage(RImage *image, int bevel_type);
 
-RImage *RRenderGradient(unsigned width, unsigned height, RColor *from,
-                        RColor *to, int style);
+RImage *RRenderGradient(unsigned width, unsigned height, const RColor *from,
+                        const RColor *to, RGradientStyle style);
 
 
 RImage *RRenderMultiGradient(unsigned width, unsigned height, RColor **colors,
-                             int style);
+                             RGradientStyle style);
 
 
 RImage *RRenderInterwovenGradient(unsigned width, unsigned height,
@@ -465,5 +502,10 @@ extern int RErrorCode;
 }
 #endif /* __cplusplus */
 
-#endif
+/*
+ * The definitions below are done for internal use only
+ * We undef them so users of the library may not misuse them
+ */
+#undef __wrlib_deprecated
 
+#endif
